/**
 * P4
 * http://snarfed.org/space/p4
 *
 * Copyright 2001-2004
 * Ken Ashcraft <kashcraft+p4@gmail.com>
 * Ryan Barrett <p4@ryanb.org>
 * Maulik Shah <p4@maulik.net>
 * Nathan Stoll <nathanstoll+p4@gmail.com>
 *
 * @file msg_data.cpp
 *
 * Defines the cmsg_data class, which is derived from the abstract cmsg class.
 * The cmsg_data class is the class that DATA messages instantiate. For more
 * information, see protocol.txt, msg.h, and protocol.h.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "msg_data.h"
#include "receiveq.h"
#include "p4error.h"

/*****************************************************************************
 * PUBLIC METHODS
 *****************************************************************************/

cmsg_data::cmsg_data(ushort plugin_id, const char *data,
                     uint len, bool encrypted) : cmsg(plugin_id, 0, len)
{
  hdr.set_type(kp4_data);
  this->data = new char[len];
  memcpy(this->data, data, len);
}


/*****************************************************************************
 * PROTECTED METHODS
 *****************************************************************************/

bool cmsg_data::virt_process() const
{
  cmsg_data *nc_this = const_cast<cmsg_data *>(this);

  if (creceiveq::instance().enqueue(nc_this) == kerr_success)
    return false;  // enqueued, return false so we don't get deleted
  else
    return true;
}
